@@ -251,9 +251,9 @@ class LEDENETDevice:
         """Retrurn the hardware mapping for the device."""
         if not self._discovery or ATTR_MODEL not in self._discovery:
             return None
-        assert self._discovery is not None
-        assert self._discovery[ATTR_MODEL] is not None
-        model = self._discovery[ATTR_MODEL]
+        model = self._discovery.get(ATTR_MODEL)
+        if model is None:
+            return None
         assert isinstance(model, str)
         return HARDWARE_MAP.get(model)
 
