@@ -119,13 +119,14 @@ BASE_MODE_MAP = {
 
 
 MULTI_MODE_NUM_TO_MODE = {
+    0: COLOR_MODE_RGBWW,  # Factory default, no mode set AKA RGBWW
     1: COLOR_MODE_DIM,
     2: COLOR_MODE_CCT,
     3: COLOR_MODE_RGB,
     4: COLOR_MODE_RGBW,
     5: COLOR_MODE_RGBWW,
 }
-MULTI_MODE_MODE_TO_NUM = {v: k for k, v in MULTI_MODE_NUM_TO_MODE.items()}
+MULTI_MODE_MODE_TO_NUM = {v: k for k, v in MULTI_MODE_NUM_TO_MODE.items() if k != 0}
 
 GENERIC_RGB_MAP = {
     0x13: {COLOR_MODE_RGB},  # RGB (RGB) verified on model 0x33
@@ -608,7 +609,7 @@ MODELS = [
     LEDENETModel(
         model_num=0x06,
         # "AK001-ZJ2104" == v1 has RF remote support
-        # "AK001-ZJ2145" == v2.03, v2.09 has IR remote support
+        # "AK001-ZJ2145" == v2.03, v2.06, v2.09 has IR remote support but not always pinned out
         # "AK001-ZJ2147" == v3.31 has 2.4ghz remote support
         models=[
             "AK001-ZJ100",
@@ -648,7 +649,7 @@ MODELS = [
     LEDENETModel(
         model_num=0x08,
         # AK001-ZJ2101 is v1
-        # AK001-ZJ2145 is v2
+        # AK001-ZJ2145 is v2.11
         models=["AK001-ZJ2101", "AK001-ZJ2145", "AK001-ZJ2147"],
         description="Controller RGB with MIC",
         always_writes_white_and_colors=True,  # Formerly rgbwprotocol
@@ -918,8 +919,7 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x41,
-        # 'AK001-ZJ2104' is v2
-        # 'AK001-ZJ2104' likely supports turning on by effect/levels set
+        # 'AK001-ZJ2104' is v2.51 and supports turning on by effect/levels set
         models=["AK001-ZJ2104"],
         description="Controller Dimmable",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
@@ -1068,7 +1068,7 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x97,  # 0x97
-        # AK001-ZJ2146 = v3 (has BLE)
+        # AK001-ZJ2146 = v3.11 (has BLE)
         models=["AK001-ZJ2134", "AK001-ZJ2146"],
         description="Socket",  # 1 channel
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
